@@ -106,7 +106,7 @@ do_join(register SV *sv, SV *del, register SV **mark, register SV **sp)
 	sv_upgrade(sv, SVt_PV);
     if (SvLEN(sv) < len + items) {	/* current length is way too short */
 	while (items-- > 0) {
-	    if (*mark) {
+	    if (*mark && !SvGMAGICAL(*mark) && SvOK(*mark)) {
 		SvPV(*mark, tmplen);
 		len += tmplen;
 	    }
@@ -474,7 +474,7 @@ do_kv(ARGSproto)
 	RETURN;
 
     if (gimme == G_SCALAR) {
-	I32 i;
+	IV i;
 	dTARGET;
 
 	if (op->op_flags & OPf_MOD) {	/* lvalue */
@@ -483,7 +483,11 @@ do_kv(ARGSproto)
 		sv_magic(TARG, Nullsv, 'k', Nullch, 0);
 	    }
 	    LvTYPE(TARG) = 'k';
-	    LvTARG(TARG) = (SV*)hv;
+	    if (LvTARG(TARG) != (SV*)hv) {
+		if (LvTARG(TARG))
+		    SvREFCNT_dec(LvTARG(TARG));
+		LvTARG(TARG) = SvREFCNT_inc(hv);
+	    }
 	    PUSHs(TARG);
 	    RETURN;
 	}
