@@ -181,9 +181,12 @@ struct pmop {
     REGEXP *	op_pmregexp;		/* compiled expression */
     U16		op_pmflags;
     U16		op_pmpermflags;
+    U8		op_pmdynflags;
 };
 
-#define PMf_USED	0x0001		/* pm has been used once already */
+#define PMdf_USED	0x01		/* pm has been used once already */
+
+#define PMf_TAINTMEM	0x0001		/* taint $1 etc. if target tainted */
 #define PMf_ONCE	0x0002		/* use pattern only once per reset */
 #define PMf_REVERSED	0x0004		/* Should be matched right->left */
 /*#define PMf_ALL		0x0008*/		/* initial constant is whole pat */
