@@ -362,7 +362,7 @@ abs		abs			ck_fun		fstu%	S?
 # String stuff.
 
 length		length			ck_lengthconst	istu%	S?
-substr		substr			ck_fun		st@	S S S?
+substr		substr			ck_fun		st@	S S S? S?
 vec		vec			ck_fun		ist@	S S S
 
 index		index			ck_index	ist@	S S S?
