@@ -91,7 +91,7 @@ register struct op *op asm(stringify(OP_IN_REGISTER));
 #define SOFT_CAST(type)	(type)
 #endif
 
-#ifndef BYTEORDER
+#ifndef BYTEORDER  /* Should never happen -- byteorder is in config.h */
 #   define BYTEORDER 0x1234
 #endif
 
@@ -695,12 +695,21 @@ Free_t   Perl_free _((Malloc_t where));
 #   ifdef convex
 #	define Quad_t long long
 #   else
-#	if BYTEORDER > 0xFFFF
+#	if LONGSIZE == 8
 #	    define Quad_t long
 #	endif
 #   endif
 #endif
 
+/* XXX Experimental set-up for long long.  Just add -DUSE_LONG_LONG
+   to your ccflags.  --Andy Dougherty   4/1998
+*/
+#ifdef USE_LONG_LONG
+#  if defined(HAS_LONG_LONG) && LONGLONGSIZE == 8
+#    define Quad_t long long
+#  endif
+#endif
+
 #ifdef Quad_t
 #   define HAS_QUAD
     typedef Quad_t IV;
@@ -1645,6 +1654,12 @@ typedef enum {
 #define PERLVARI(var,type,init) type var;
 #define PERLVARIC(var,type,init) type var;
 
+/* Interpreter exitlist entry */
+typedef struct exitlistentry {
+    void (*fn) _((void*));
+    void *ptr;
+} PerlExitListEntry;
+
 #ifdef PERL_GLOBAL_STRUCT
 struct perl_vars {
 #include "perlvars.h"
@@ -1794,13 +1809,15 @@ EXT MGVTBL vtbl_glob =	{magic_getglob,
 					0,	0,	0};
 EXT MGVTBL vtbl_mglob =	{0,	magic_setmglob,
 					0,	0,	0};
-EXT MGVTBL vtbl_nkeys =	{0,	magic_setnkeys,
+EXT MGVTBL vtbl_nkeys =	{magic_getnkeys,
+				magic_setnkeys,
 					0,	0,	0};
 EXT MGVTBL vtbl_taint =	{magic_gettaint,magic_settaint,
 					0,	0,	0};
-EXT MGVTBL vtbl_substr =	{0,	magic_setsubstr,
+EXT MGVTBL vtbl_substr =	{magic_getsubstr, magic_setsubstr,
 					0,	0,	0};
-EXT MGVTBL vtbl_vec =	{0,	magic_setvec,
+EXT MGVTBL vtbl_vec =	{magic_getvec,
+				magic_setvec,
 					0,	0,	0};
 EXT MGVTBL vtbl_pos =	{magic_getpos,
 				magic_setpos,
@@ -2003,7 +2020,7 @@ enum {
 
 #endif /* OVERLOAD */
 
-#define PERLDB_ALL	0xff
+#define PERLDB_ALL	0x3f		/* No _NONAME, _GOTO */
 #define PERLDBf_SUB	0x01		/* Debug sub enter/exit. */
 #define PERLDBf_LINE	0x02		/* Keep line #. */
 #define PERLDBf_NOOPT	0x04		/* Switch off optimizations. */
@@ -2011,6 +2028,8 @@ enum {
 					   later inspections.  */
 #define PERLDBf_SUBLINE	0x10		/* Keep subr source lines. */
 #define PERLDBf_SINGLE	0x20		/* Start with single-step on. */
+#define PERLDBf_NONAME	0x40		/* For _SUB: no name of the subr. */
+#define PERLDBf_GOTO	0x80		/* Report goto: call DB::goto. */
 
 #define PERLDB_SUB	(perldb && (perldb & PERLDBf_SUB))
 #define PERLDB_LINE	(perldb && (perldb & PERLDBf_LINE))
@@ -2018,6 +2037,8 @@ enum {
 #define PERLDB_INTER	(perldb && (perldb & PERLDBf_INTER))
 #define PERLDB_SUBLINE	(perldb && (perldb & PERLDBf_SUBLINE))
 #define PERLDB_SINGLE	(perldb && (perldb & PERLDBf_SINGLE))
+#define PERLDB_SUB_NN	(perldb && (perldb & (PERLDBf_NONAME)))
+#define PERLDB_GOTO	(perldb && (perldb & PERLDBf_GOTO))
 
 
 #ifdef USE_LOCALE_NUMERIC
@@ -2063,6 +2084,9 @@ enum {
 	    nice_chunk = (char*)(chunk);		\
 	    nice_chunk_size = (chunk_size);		\
 	}						\
+	else {						\
+	    Safefree(chunk);				\
+	}						\
 	UNLOCK_SV_MUTEX;				\
     } while (0)
 
